#include <libsegy.h>
#include <mex.h>
#include <matrix.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// mexfunction(out_argc, out_argv, in_argc, in_argv)
//   out_argc: int = number of output arguments requested by caller
//   out_argv: mxArray * = hooks to assign output args to
//   in_argc : int = number of input arguments assigned by caller
//   in_argv : mxArray * = input argument pointers
//
//   in_argv[0] : char array  = input ascii data
//   out_argv[0]: char array  = output ebcdic (codepage 00037) data
//
void mexFunction(int out_argc,  mxArray *out_argv[], 
                 int in_argc, const mxArray *in_argv[])
{
  int in_bytes = 0; int i;
  mwSize size[2] = {0, 0};
  char *buf = NULL; short *in = NULL; short *out = NULL;

  // check input arguments
  if (in_argc < 1 || ! mxIsChar(in_argv[0])) {
    mexPrintf("segy_ebcdic2ascii: input must be array of type char\n");
    return;
  }

  if (out_argc < 1) {
    mexPrintf("segy_ebcdic2ascii: must have one output argument\n");
    return;
  }

  in_bytes = mxGetNumberOfElements(in_argv[0]);
  if (in_bytes < 1) {
    mexPrintf("segy_ebcdic2ascii: input array is empty\n");  
    return;
  }
  
  if (! mxGetData(in_argv[0])) {
    mexPrintf("segy_ebcdic2ascii: error reading input data\n");
    return;
  }
  in = (short*)mxGetData(in_argv[0]);

  // copy input to buffer and convert to ascii
  buf = (char*)malloc(in_bytes);
  for (i = 0; i < in_bytes; i++) buf[i] = in[i];
  segy_ascii2ebcdic(buf, in_bytes);

  // copy to output arg
  size[0] = 1; size[1] = in_bytes;
  if (! (out_argv[0] = mxCreateCharArray(2, size)) ) {
    mexPrintf("segy_ebcdic2ascii: error creating output, out of memory\n");
    return;
  }
  if (! mxGetData(out_argv[0])) {
    mexPrintf("segy_ebcdic2ascii: error converting output to type char\n");
    return;
  }
  out = (short*)mxGetData(out_argv[0]);
  for (i = 0; i < in_bytes; i++) out[i] = buf[i];

  free(buf);
}
