#include <libsegy.h>
#include "segy_matlab.h"
#include <mex.h>
#include <matrix.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

// mexfunction(out_argc, out_argv, in_argc, in_argv)
//   out_argc: int = number of output arguments requested by caller
//   out_argv: mxArray * = hooks to assign output args to
//   in_argc : int = number of input arguments assigned by caller
//   in_argv : mxArray * = input argument pointers
//
//   out_argv[0]: struct = MATLAB struct with properly formatted fields
//
void mexFunction(int out_argc,  mxArray *out_argv[], 
                 int in_argc, const mxArray *in_argv[])
{
  mwSize size[2] = {0,0};

  // check ouput arguments
  if (out_argc < 1) {
    mexPrintf("segy_new_file_header: must have one output argument\n");
    return;
  }

  // create struct with fieldnames
  out_argv[0] = mxCreateStructMatrix(1, 1, 33, segy_file_header_names);
  if (! out_argv[0]) {
    mexPrintf("segy_new_file_header: out of memory\n");
    return;
  }

  // fill in default values of the proper type
  size[0] = 1; size[1] = 3200;
  mxSetField(out_argv[0], 0, "TextHeader", mxCreateCharArray(2, size));

  mxSetField(out_argv[0], 0, "JobID", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "LineNumber", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "ReelNumber", mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "TracesPerRecord", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "AuxTracesPerRecord", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "ReelSampleInterval", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "FieldSampleInterval", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "ReelSamplesPerTrace", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "FieldSamplesPerTrace", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SampleFormatCode", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "CDPFold", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "TraceSortingCode", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "VertSumCode", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "StartSweepFreq", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "EndSweepFreq", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SweepLength", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SweepCode", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "SweepTraceNumber", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "StartSweepTaperLength", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "EndSweepTaperLength", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "TaperType", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "DataTracesCorrelated", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "BinaryGainRecovered", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "AmpRecoverMethod", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "MeasurementSystem", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "ImpulseSignalPolarity", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "VibratoryPolarityCode", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));


  size[0] = 1; size[1] = 240;
  mxSetField(out_argv[0], 0, "unassigned", mxCreateCharArray(2, size));

  mxSetField(out_argv[0], 0, "RevisionNumber", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "FixedLengthTraceFlag", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  mxSetField(out_argv[0], 0, "ExtendedTextHeaders", mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL));

  size[0] = 1; size[1] = 94;
  mxSetField(out_argv[0], 0, "unassigned2", mxCreateCharArray(2, size));
}
