#include <libsegy.h>
#include "segy_matlab.h"
#include <mex.h>
#include <matrix.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


// mexfunction(out_argc, out_argv, in_argc, in_argv)
//   out_argc: int = number of output arguments requested by caller
//   out_argv: mxArray * = hooks to assign output args to
//   in_argc : int = number of input arguments assigned by caller
//   in_argv : mxArray * = input argument pointers
//
//   in_argv[0] : int   = open file descriptor
//   in_argv[1] : short = number of floats to read from trace data
//   out_argv[0]: int = error code, 0 on success, nonzero on error
//   out_argv[1]: float vector = file trace data
//
void mexFunction(int out_argc,  mxArray *out_argv[], 
                 int in_argc, const mxArray *in_argv[])
{
  int fd; int err = 0; short len = 0; int i;
  float *data = NULL;

  // check input arguments
  if (in_argc < 2 || ! mxIsNumeric(in_argv[0]) || ! mxIsNumeric(in_argv[1]) 
      || mxIsEmpty(in_argv[0]) || mxIsEmpty(in_argv[1])) {
    mexPrintf("segy_read_data: input is 2 int arguments\n");
    return;
  }
  fd = (int)mxGetScalar(in_argv[0]);
  len = (int)mxGetScalar(in_argv[1]);
  data = (float*)malloc(sizeof(float) * len);

  // check output arguments
  if (out_argc < 2) {
    mexPrintf("segy_read_data: must have two output arguments\n");
    return;
  }
  
  // call function, if error return immeadiately
  err = segy_read_data(fd, 1, len, data, len);

  // store error return
  out_argv[0] = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (out_argv[0]) ((int*)mxGetPr(out_argv[0]))[0] = err;
  else {
    mexPrintf("segy_read_data: out of memory\n");
    return;
  }

  if (err) {
    // store zeroed data array
    out_argv[1] = mxCreateNumericMatrix(len,1,mxSINGLE_CLASS,mxREAL);
    if (! out_argv[1]) {
      mexPrintf("segy_read_data: out of memory\n");
      return;
    }

    free(data);
    return;
  }

  // create output MATLAB data array
  out_argv[1] = mxCreateNumericMatrix(len,1,mxSINGLE_CLASS,mxREAL);
  if (! out_argv[1]) {
    mexPrintf("segy_read_data: out of memory\n");
    return;
  }

  // store data in data array
  for (i=0; i<len; i++) ((float*)mxGetPr(out_argv[1]))[i] = data[i];
  free(data);
}
