#include <libsegy.h>
#include "segy_matlab.h"
#include <mex.h>
#include <matrix.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>


// mexfunction(out_argc, out_argv, in_argc, in_argv)
//   out_argc: int = number of output arguments requested by caller
//   out_argv: mxArray * = hooks to assign output args to
//   in_argc : int = number of input arguments assigned by caller
//   in_argv : mxArray * = input argument pointers
//
//   in_argv[0] : int = open file descriptor
//   out_argv[0]: int    = error code, 0 if no errors
//   out_argv[1]: struct = file header data
//
void mexFunction(int out_argc,  mxArray *out_argv[], 
                 int in_argc, const mxArray *in_argv[])
{
  int fd = 0; int err = 0; int i;
  segy_file_header header;
  mwSize size[2] = {0, 0};
  mxArray *TextHeader, *JobID, *LineNumber, *ReelNumber;
  mxArray *TracesPerRecord, *AuxTracesPerRecord, *ReelSampleInterval, *FieldSampleInterval;
  mxArray *ReelSamplesPerTrace, *FieldSamplesPerTrace, *SampleFormatCode, *CDPFold;
  mxArray *TraceSortingCode, *VertSumCode, *StartSweepFreq, *EndSweepFreq;
  mxArray *SweepLength, *SweepCode, *SweepTraceNumber, *StartSweepTaperLength;
  mxArray *EndSweepTaperLength, *TaperType, *DataTracesCorrelated, *BinaryGainRecovered;
  mxArray *AmpRecoverMethod, *MeasurementSystem, *ImpulseSignalPolarity, *VibratoryPolarityCode;
  mxArray *unassigned, *RevisionNumber, *FixedLengthTraceFlag, *ExtendedTextHeaders;
  mxArray *unassigned2;

  // check input arguments
  if (in_argc < 1 || ! mxIsInt32(in_argv[0]) || mxIsEmpty(in_argv[0])) {
    mexPrintf("segy_read_file_header: input must be an open file descriptor\n");
    return;
  }
  fd = ((int*)mxGetPr(in_argv[0]))[0];

  // check output arguments
  if (out_argc < 2) {
    mexPrintf("segy_read_file_header: must have two output arguments\n");
    return;
  }

  // call function, if error return immeadiately
  err = segy_read_file_header(fd, &header);

  out_argv[0] = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (out_argv[0]) ((int*)mxGetPr(out_argv[0]))[0] = err;
  else {
    mexPrintf("segy_read_file_header: out of memory\n");
    return;
  }

  if (err) {
    size[0] = 1; size[1] = 1;
    out_argv[1] = mxCreateStructArray(2, size, 0, NULL);
    if (! out_argv[1]) {
      mexPrintf("segy_read_file_header: out of memory\n");
      return;
    }
    return;
  }

  // create output MATLAB struct
  size[0] = 1; size[1] = 1;
  out_argv[1] = mxCreateStructArray(2, size, 33, segy_file_header_names);
  if (! out_argv[1]) {
    mexPrintf("segy_read_file_header: out of memory\n");
    return;
  }
  
  // copy C struct field values to MATLAB struct fields
  size[0] = 1; size[1] = 3200;
  TextHeader = mxCreateCharArray(2, size);
  if (! TextHeader) {
    mexPrintf("out of memory");
    return;
  }
  for (i=0; i<3200; i++) 
    ((short*)mxGetPr(TextHeader))[i] = header.TextHeader[i];
  mxSetField(out_argv[1], 0, "TextHeader", TextHeader);

  JobID = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(JobID)) ((int*)mxGetData(JobID))[0] = header.JobID;
  mxSetField(out_argv[1], 0, "JobID", JobID);

  LineNumber = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(LineNumber)) ((int*)mxGetData(LineNumber))[0] = header.LineNumber;
  mxSetField(out_argv[1], 0, "LineNumber", LineNumber);

  ReelNumber = mxCreateNumericMatrix(1,1,mxINT32_CLASS,mxREAL);
  if (mxGetData(ReelNumber)) ((int*)mxGetData(ReelNumber))[0] = header.ReelNumber;
  mxSetField(out_argv[1], 0, "ReelNumber", ReelNumber);

  TracesPerRecord = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(TracesPerRecord)) ((short*)mxGetData(TracesPerRecord))[0] = header.TracesPerRecord;
  mxSetField(out_argv[1], 0, "TracesPerRecord", TracesPerRecord);

  AuxTracesPerRecord = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(AuxTracesPerRecord)) ((short*)mxGetData(AuxTracesPerRecord))[0] = header.AuxTracesPerRecord;
  mxSetField(out_argv[1], 0, "AuxTracesPerRecord", AuxTracesPerRecord);

  ReelSampleInterval = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(ReelSampleInterval)) ((short*)mxGetData(ReelSampleInterval))[0] = header.ReelSampleInterval;
  mxSetField(out_argv[1], 0, "ReelSampleInterval", ReelSampleInterval);

  FieldSampleInterval = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(FieldSampleInterval)) ((short*)mxGetData(FieldSampleInterval))[0] = header.FieldSampleInterval;
  mxSetField(out_argv[1], 0, "FieldSampleInterval", FieldSampleInterval);

  ReelSamplesPerTrace = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(ReelSamplesPerTrace)) ((short*)mxGetData(ReelSamplesPerTrace))[0] = header.ReelSamplesPerTrace;
  mxSetField(out_argv[1], 0, "ReelSamplesPerTrace", ReelSamplesPerTrace);

  FieldSamplesPerTrace = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(FieldSamplesPerTrace)) ((short*)mxGetData(FieldSamplesPerTrace))[0] = header.FieldSamplesPerTrace;
  mxSetField(out_argv[1], 0, "FieldSamplesPerTrace", FieldSamplesPerTrace);

  SampleFormatCode = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SampleFormatCode)) ((short*)mxGetData(SampleFormatCode))[0] = header.SampleFormatCode;
  mxSetField(out_argv[1], 0, "SampleFormatCode", SampleFormatCode);

  CDPFold = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(CDPFold)) ((short*)mxGetData(CDPFold))[0] = header.CDPFold;
  mxSetField(out_argv[1], 0, "CDPFold", CDPFold);

  TraceSortingCode = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(TraceSortingCode)) ((short*)mxGetData(TraceSortingCode))[0] = header.TraceSortingCode;
  mxSetField(out_argv[1], 0, "TraceSortingCode", TraceSortingCode);

  VertSumCode = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(VertSumCode)) ((short*)mxGetData(VertSumCode))[0] = header.VertSumCode;
  mxSetField(out_argv[1], 0, "VertSumCode", VertSumCode);

  StartSweepFreq = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(StartSweepFreq)) ((short*)mxGetData(StartSweepFreq))[0] = header.StartSweepFreq;
  mxSetField(out_argv[1], 0, "StartSweepFreq", StartSweepFreq);

  EndSweepFreq = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(EndSweepFreq)) ((short*)mxGetData(EndSweepFreq))[0] = header.EndSweepFreq;
  mxSetField(out_argv[1], 0, "EndSweepFreq", EndSweepFreq);

  SweepLength = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SweepLength)) ((short*)mxGetData(SweepLength))[0] = header.SweepLength;
  mxSetField(out_argv[1], 0, "SweepLength", SweepLength);

  SweepCode = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SweepCode)) ((short*)mxGetData(SweepCode))[0] = header.SweepCode;
  mxSetField(out_argv[1], 0, "SweepCode", SweepCode);

  SweepTraceNumber = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(SweepTraceNumber)) ((short*)mxGetData(SweepTraceNumber))[0] = header.SweepTraceNumber;
  mxSetField(out_argv[1], 0, "SweepTraceNumber", SweepTraceNumber);

  StartSweepTaperLength = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(StartSweepTaperLength)) ((short*)mxGetData(StartSweepTaperLength))[0] = header.StartSweepTaperLength;
  mxSetField(out_argv[1], 0, "StartSweepTaperLength", StartSweepTaperLength);

  EndSweepTaperLength = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(EndSweepTaperLength)) ((short*)mxGetData(EndSweepTaperLength))[0] = header.EndSweepTaperLength;
  mxSetField(out_argv[1], 0, "EndSweepTaperLength", EndSweepTaperLength);

  TaperType = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(TaperType)) ((short*)mxGetData(TaperType))[0] = header.TaperType;
  mxSetField(out_argv[1], 0, "TaperType", TaperType);

  DataTracesCorrelated = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(DataTracesCorrelated)) ((short*)mxGetData(DataTracesCorrelated))[0] = header.DataTracesCorrelated;
  mxSetField(out_argv[1], 0, "DataTracesCorrelated", DataTracesCorrelated);

  BinaryGainRecovered = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(BinaryGainRecovered)) ((short*)mxGetData(BinaryGainRecovered))[0] = header.BinaryGainRecovered;
  mxSetField(out_argv[1], 0, "BinaryGainRecovered", BinaryGainRecovered);

  AmpRecoverMethod = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(AmpRecoverMethod)) ((short*)mxGetData(AmpRecoverMethod))[0] = header.AmpRecoverMethod;
  mxSetField(out_argv[1], 0, "AmpRecoverMethod", AmpRecoverMethod);

  MeasurementSystem = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(MeasurementSystem)) ((short*)mxGetData(MeasurementSystem))[0] = header.MeasurementSystem;
  mxSetField(out_argv[1], 0, "MeasurementSystem", MeasurementSystem);

  ImpulseSignalPolarity = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(ImpulseSignalPolarity)) ((short*)mxGetData(ImpulseSignalPolarity))[0] = header.ImpulseSignalPolarity;
  mxSetField(out_argv[1], 0, "ImpulseSignalPolarity", ImpulseSignalPolarity);

  VibratoryPolarityCode = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(VibratoryPolarityCode)) ((short*)mxGetData(VibratoryPolarityCode))[0] = header.VibratoryPolarityCode;
  mxSetField(out_argv[1], 0, "VibratoryPolarityCode", VibratoryPolarityCode);

  size[0] = 1; size[1] = 240;
  unassigned = mxCreateCharArray(2, size);
  if (! unassigned) {
    mexPrintf("out of memory");
    return;
  }
  for (i=0; i<240; i++) 
    ((short*)mxGetPr(unassigned))[i] = header.unassigned[i];
  mxSetField(out_argv[1], 0, "unassigned", unassigned);

  RevisionNumber = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(RevisionNumber)) ((short*)mxGetData(RevisionNumber))[0] = header.RevisionNumber;
  mxSetField(out_argv[1], 0, "RevisionNumber", RevisionNumber);

  FixedLengthTraceFlag = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(FixedLengthTraceFlag)) ((short*)mxGetData(FixedLengthTraceFlag))[0] = header.FixedLengthTraceFlag;
  mxSetField(out_argv[1], 0, "FixedLengthTraceFlag", FixedLengthTraceFlag);

  ExtendedTextHeaders = mxCreateNumericMatrix(1,1,mxINT16_CLASS,mxREAL);
  if (mxGetData(ExtendedTextHeaders)) ((short*)mxGetData(ExtendedTextHeaders))[0] = header.ExtendedTextHeaders;
  mxSetField(out_argv[1], 0, "ExtendedTextHeaders", ExtendedTextHeaders);

  size[0] = 1; size[1] = 94;
  unassigned2 = mxCreateCharArray(2, size);
  if (! unassigned2) {
    mexPrintf("out of memory");
    return;
  }
  for (i=0; i<94; i++) 
    ((short*)mxGetPr(unassigned2))[i] = header.unassigned2[i];
  mxSetField(out_argv[1], 0, "unassigned2", unassigned2);
}
