#include <stdio.h>      // printf
#include <sys/types.h>  // open
#include <sys/stat.h>   // open
#include <fcntl.h>      // open
#include <unistd.h>     // close
#include <stdlib.h>     // atoi
#include <libsegy.h>

#ifndef O_BINARY
#define O_BINARY 0
#endif

static char hex[9] = {0};

const char * printbyte(char *byte) 
{
  char lo = (*byte) & 0x0f;
  char hi = ((*byte) >> 4) & 0x0f;
  if (lo < 10) lo += 48;
  else lo += 87;
  if (hi < 10) hi += 48;
  else hi += 87;
  hex[0] = hi;
  hex[1] = lo;
  hex[2] = 0;
  return hex;
}

const char * printshort(short *s) 
{
  char *sc = (char*)s;
  char result[5] = {0};
  snprintf(&result[0], 3, "%s", printbyte(&sc[0]));
  snprintf(&result[2], 3, "%s", printbyte(&sc[1]));
  snprintf(hex, 5, "%s", result);
  return hex;
}

const char * printint(int *i) 
{
  char *ci = (char*)i;
  char result[9] = {0};
  snprintf(&result[0], 3, "%s", printbyte(&ci[0]));
  snprintf(&result[2], 3, "%s", printbyte(&ci[1]));
  snprintf(&result[4], 3, "%s", printbyte(&ci[2]));
  snprintf(&result[6], 3, "%s", printbyte(&ci[3]));
  snprintf(hex, 9, "%s", result);
  return hex;
}

const char * printfloat(float *f) 
{
  char *cf = (char*)f;
  char result[9] = {0};
  snprintf(&result[0], 3, "%s", printbyte(&cf[0]));
  snprintf(&result[2], 3, "%s", printbyte(&cf[1]));
  snprintf(&result[4], 3, "%s", printbyte(&cf[2]));
  snprintf(&result[6], 3, "%s", printbyte(&cf[3]));
  snprintf(hex, 9, "%s", result);
  return hex;
}


int main(int argc, char **argv)
{
  int err = 0; int i;
  char *USAGE="USAGE: printsegy segy_file\n";
  if (argc < 2) {
    printf(USAGE);
    return 1;
  }

  // open files
  int fd = open(argv[1], O_RDONLY | O_BINARY);
  if (fd < 0) {
    printf("libsegy: Error opening file %s\n", argv[1]);
    return 1;
  }

  // get trace limit
  int limit = 0;
  if (argc >= 3) {
    limit = atoi(argv[2]);
  }
  if (limit < 0) limit = 0;

  // read file header
  segy_file_header fhead;
  if ( (err = segy_read_file_header(fd, &fhead)) ) {
    printf("%s\n", segy_error_message(err));
    return 1;
  }

  // print file text header
  printf("Text Header:\n");
  for (i = 0; i < 3200; i++) {
    printf("%c", fhead.TextHeader[i]);
    if ((i+1) % 80 == 0) printf("\n");
  }
  printf("\n");

  // print file binary header
  printf("Binary File Header:\n");
  printf("%-25s: %5d (%s)\n", "JobID",
         fhead.JobID, printint(&fhead.JobID));

  printf("%-25s: %5d (%s)\n", "LineNumber",
         fhead.LineNumber, printint(&fhead.LineNumber));

  printf("%-25s: %5d (%s)\n", "ReelNumber",
         fhead.ReelNumber, printint(&fhead.ReelNumber));

  printf("%-25s: %5d (%s)\n", "TracesPerRecord",
         fhead.TracesPerRecord, printshort(&fhead.TracesPerRecord));

  printf("%-25s: %5d (%s)\n", "AuxTracesPerRecord",
         fhead.AuxTracesPerRecord, printshort(&fhead.AuxTracesPerRecord));

  printf("%-25s: %5d (%s)\n", "ReelSampleInterval",
         fhead.ReelSampleInterval, printshort(&fhead.ReelSampleInterval));

  printf("%-25s: %5d (%s)\n", "FieldSampleInterval",
         fhead.FieldSampleInterval, printshort(&fhead.FieldSampleInterval));

  printf("%-25s: %5d (%s)\n", "ReelSamplesPerTrace",
         fhead.ReelSamplesPerTrace, printshort(&fhead.ReelSamplesPerTrace));

  printf("%-25s: %5d (%s)\n", "FieldSamplesPerTrace",
         fhead.FieldSamplesPerTrace, printshort(&fhead.FieldSamplesPerTrace));

  printf("%-25s: %5d (%s)\n", "SampleFormatCode",
         fhead.SampleFormatCode, printshort(&fhead.SampleFormatCode));

  printf("%-25s: %5d (%s)\n", "CDPFold",
         fhead.CDPFold, printshort(&fhead.CDPFold));

  printf("%-25s: %5d (%s)\n", "TraceSortingCode",
         fhead.TraceSortingCode, printshort(&fhead.TraceSortingCode));

  printf("%-25s: %5d (%s)\n", "VertSumCode",
         fhead.VertSumCode, printshort(&fhead.VertSumCode));

  printf("%-25s: %5d (%s)\n", "StartSweepFreq",
         fhead.StartSweepFreq, printshort(&fhead.StartSweepFreq));

  printf("%-25s: %5d (%s)\n", "EndSweepFreq",
         fhead.EndSweepFreq, printshort(&fhead.EndSweepFreq));

  printf("%-25s: %5d (%s)\n", "SweepLength",
         fhead.SweepLength, printshort(&fhead.SweepLength));

  printf("%-25s: %5d (%s)\n", "SweepCode",
         fhead.SweepCode, printshort(&fhead.SweepCode));

  printf("%-25s: %5d (%s)\n", "SweepTraceNumber",
         fhead.SweepTraceNumber, printshort(&fhead.SweepTraceNumber));

  printf("%-25s: %5d (%s)\n", "StartSweepTaperLength",
         fhead.StartSweepTaperLength, printshort(&fhead.StartSweepTaperLength));

  printf("%-25s: %5d (%s)\n", "EndSweepTaperLength",
         fhead.EndSweepTaperLength, printshort(&fhead.EndSweepTaperLength));

  printf("%-25s: %5d (%s)\n", "TaperType",
         fhead.TaperType, printshort(&fhead.TaperType));

  printf("%-25s: %5d (%s)\n", "DataTracesCorrelated",
         fhead.DataTracesCorrelated, printshort(&fhead.DataTracesCorrelated));

  printf("%-25s: %5d (%s)\n", "BinaryGainRecovered",
         fhead.BinaryGainRecovered, printshort(&fhead.BinaryGainRecovered));

  printf("%-25s: %5d (%s)\n", "AmpRecoverMethod",
         fhead.AmpRecoverMethod, printshort(&fhead.AmpRecoverMethod));

  printf("%-25s: %5d (%s)\n", "MeasurementSystem",
         fhead.MeasurementSystem, printshort(&fhead.MeasurementSystem));

  printf("%-25s: %5d (%s)\n", "ImpulseSignalPolarity",
         fhead.ImpulseSignalPolarity, printshort(&fhead.ImpulseSignalPolarity));

  printf("%-25s: %5d (%s)\n", "VibratoryPolarityCode",
         fhead.VibratoryPolarityCode, printshort(&fhead.VibratoryPolarityCode));

  printf("%-25s:  ", "unassigned[240]");
  for (i = 0; i < 240 && i < 40; i++) {
    if (i == 0) printf("'");
    printf("%hhx", fhead.unassigned[i]);
    if (i == 39) printf("...'\n");
  }

  printf("%-25s: %5d (%s)\n", "RevisionNumber",
         fhead.RevisionNumber, printshort(&fhead.RevisionNumber));

  printf("%-25s: %5d (%s)\n", "FixedLengthTraceFlag",
         fhead.FixedLengthTraceFlag, printshort(&fhead.FixedLengthTraceFlag));

  printf("%-25s: %5d (%s)\n", "ExtendedTextHeaders",
         fhead.ExtendedTextHeaders, printshort(&fhead.ExtendedTextHeaders));

  printf("%-25s:  ", "unassigned2[94]");
  for (i = 0; i < 94 && i < 40; i++) {
    if (i == 0) printf("'");
    printf("%hhx", fhead.unassigned2[i]);
    if (i == 39) printf("...'\n");
  }
  printf(" <%9d>\n\n", (int)lseek(fd, 0, SEEK_CUR));

  // print trace header and data
  int trace_num = 0;
  segy_trace_header thead;
  float data[1024] = {0};
  while (err == 0) {
    if ( (err = segy_read_trace(fd, 1, &thead, data, 1024)) && 
	 err != SEGY_EOF) {
      printf("%s\n", segy_error_message(err));
      close(fd);
      return 1;
    }
    if (err == SEGY_EOF) break;
    trace_num++;

    printf("Trace Header %5d:\n", trace_num);
    printf("%-25s: %5d (%s)\n", "LineSeqNumber",
	   thead.LineSeqNumber, printint(&thead.LineSeqNumber));

    printf("%-25s: %5d (%s)\n", "ReelSeqNumber",
	   thead.ReelSeqNumber, printint(&thead.ReelSeqNumber));

    printf("%-25s: %5d (%s)\n", "FieldRecordNumber",
	   thead.FieldRecordNumber, printint(&thead.FieldRecordNumber));

    printf("%-25s: %5d (%s)\n", "TraceNumber",
	   thead.TraceNumber, printint(&thead.TraceNumber));

    printf("%-25s: %5d (%s)\n", "EnergySourceNumber",
	   thead.EnergySourceNumber, printint(&thead.EnergySourceNumber));

    printf("%-25s: %5d (%s)\n", "CDPEnsembleNumber",
	   thead.CDPEnsembleNumber, printint(&thead.CDPEnsembleNumber));

    printf("%-25s: %5d (%s)\n", "CDPTraceNumber",
	   thead.CDPTraceNumber, printint(&thead.CDPTraceNumber));

    printf("%-25s: %5d (%s)\n", "TraceCode",
	   thead.TraceCode, printshort(&thead.TraceCode));

    printf("%-25s: %5d (%s)\n", "VertSumTraces",
	   thead.VertSumTraces, printshort(&thead.VertSumTraces));

    printf("%-25s: %5d (%s)\n", "HorizSumTraces",
	   thead.HorizSumTraces, printshort(&thead.HorizSumTraces));

    printf("%-25s: %5d (%s)\n", "DataUse",
	   thead.DataUse, printshort(&thead.DataUse));

    printf("%-25s: %5d (%s)\n", "SourceGroupDist",
	   thead.SourceGroupDist, printint(&thead.SourceGroupDist));

    printf("%-25s: %5d (%s)\n", "GroupElevation",
	   thead.GroupElevation, printint(&thead.GroupElevation));

    printf("%-25s: %5d (%s)\n", "SourceElevation",
	   thead.SourceElevation, printint(&thead.SourceElevation));

    printf("%-25s: %5d (%s)\n", "SourceDepth",
	   thead.SourceDepth, printint(&thead.SourceDepth));

    printf("%-25s: %5d (%s)\n", "GroupDatumElevation",
	   thead.GroupDatumElevation, printint(&thead.GroupDatumElevation));

    printf("%-25s: %5d (%s)\n", "SourceDatumElevation",
	   thead.SourceDatumElevation, printint(&thead.SourceDatumElevation));

    printf("%-25s: %5d (%s)\n", "SourceWaterDepth",
	   thead.SourceWaterDepth, printint(&thead.SourceWaterDepth));

    printf("%-25s: %5d (%s)\n", "GroupWaterDepth",
	   thead.GroupWaterDepth, printint(&thead.GroupWaterDepth));

    printf("%-25s: %5d (%s)\n", "ElevationScalar",
	   thead.ElevationScalar, printshort(&thead.ElevationScalar));

    printf("%-25s: %5d (%s)\n", "CoordinateScalar",
	   thead.CoordinateScalar, printshort(&thead.CoordinateScalar));

    printf("%-25s: %5d (%s)\n", "SourceX",
	   thead.SourceX, printint(&thead.SourceX));

    printf("%-25s: %5d (%s)\n", "SourceY",
	   thead.SourceY, printint(&thead.SourceY));

    printf("%-25s: %5d (%s)\n", "GroupX",
	   thead.GroupX, printint(&thead.GroupX));

    printf("%-25s: %5d (%s)\n", "GroupY",
	   thead.GroupY, printint(&thead.GroupY));

    printf("%-25s: %5d (%s)\n", "CoordinateUnit",
	   thead.CoordinateUnit, printshort(&thead.CoordinateUnit));

    printf("%-25s: %5d (%s)\n", "WeatheringVelocity",
	   thead.WeatheringVelocity, printshort(&thead.WeatheringVelocity));

    printf("%-25s: %5d (%s)\n", "SubweatheringVelocity",
	   thead.SubweatheringVelocity, printshort(&thead.SubweatheringVelocity));

    printf("%-25s: %5d (%s)\n", "SourceTime",
	   thead.SourceTime, printshort(&thead.SourceTime));

    printf("%-25s: %5d (%s)\n", "GroupTime",
	   thead.GroupTime, printshort(&thead.GroupTime));

    printf("%-25s: %5d (%s)\n", "SourceCorrection",
	   thead.SourceCorrection, printshort(&thead.SourceCorrection));

    printf("%-25s: %5d (%s)\n", "GroupCorrection",
	   thead.GroupCorrection, printshort(&thead.GroupCorrection));

    printf("%-25s: %5d (%s)\n", "TotalCorrection",
	   thead.TotalCorrection, printshort(&thead.TotalCorrection));

    printf("%-25s: %5d (%s)\n", "LagTimeA",
	   thead.LagTimeA, printshort(&thead.LagTimeA));

    printf("%-25s: %5d (%s)\n", "LagTimeB",
	   thead.LagTimeB, printshort(&thead.LagTimeB));

    printf("%-25s: %5d (%s)\n", "DelayAccordingTime",
	   thead.DelayAccordingTime, printshort(&thead.DelayAccordingTime));

    printf("%-25s: %5d (%s)\n", "BruteTime",
	   thead.BruteTime, printshort(&thead.BruteTime));

    printf("%-25s: %5d (%s)\n", "MuteTime",
	   thead.MuteTime, printshort(&thead.MuteTime));

    printf("%-25s: %5d (%s)\n", "SamplesPerTrace",
	   thead.SamplesPerTrace, printshort(&thead.SamplesPerTrace));

    printf("%-25s: %5d (%s)\n", "SampleInterval",
	   thead.SampleInterval, printshort(&thead.SampleInterval));

    printf("%-25s: %5d (%s)\n", "GainType",
	   thead.GainType, printshort(&thead.GainType));

    printf("%-25s: %5d (%s)\n", "GainConstant",
	   thead.GainConstant, printshort(&thead.GainConstant));

    printf("%-25s: %5d (%s)\n", "InitialGain",
	   thead.InitialGain, printshort(&thead.InitialGain));

    printf("%-25s: %5d (%s)\n", "Correlated",
	   thead.Correlated, printshort(&thead.Correlated));

    printf("%-25s: %5d (%s)\n", "StartSweepFreq",
	   thead.StartSweepFreq, printshort(&thead.StartSweepFreq));

    printf("%-25s: %5d (%s)\n", "EndSweepFreq",
	   thead.EndSweepFreq, printshort(&thead.EndSweepFreq));

    printf("%-25s: %5d (%s)\n", "SweepLength",
	   thead.SweepLength, printshort(&thead.SweepLength));

    printf("%-25s: %5d (%s)\n", "SweepType",
	   thead.SweepType, printshort(&thead.SweepType));

    printf("%-25s: %5d (%s)\n", "StartSweepTaperLength",
	   thead.StartSweepTaperLength, printshort(&thead.StartSweepTaperLength));

    printf("%-25s: %5d (%s)\n", "EndSweepTaperLength",
	   thead.EndSweepTaperLength, printshort(&thead.EndSweepTaperLength));

    printf("%-25s: %5d (%s)\n", "TaperType",
	   thead.TaperType, printshort(&thead.TaperType));

    printf("%-25s: %5d (%s)\n", "AliasFilterFreq",
	   thead.AliasFilterFreq, printshort(&thead.AliasFilterFreq));

    printf("%-25s: %5d (%s)\n", "AliasFilterSlope",
	   thead.AliasFilterSlope, printshort(&thead.AliasFilterSlope));

    printf("%-25s: %5d (%s)\n", "NotchFilterFreq",
	   thead.NotchFilterFreq, printshort(&thead.NotchFilterFreq));

    printf("%-25s: %5d (%s)\n", "NotchFilterSlope",
	   thead.NotchFilterSlope, printshort(&thead.NotchFilterSlope));

    printf("%-25s: %5d (%s)\n", "LowCutFreq",
	   thead.LowCutFreq, printshort(&thead.LowCutFreq));

    printf("%-25s: %5d (%s)\n", "HightCutFreq",
	   thead.HightCutFreq, printshort(&thead.HightCutFreq));

    printf("%-25s: %5d (%s)\n", "LowCutSlope",
	   thead.LowCutSlope, printshort(&thead.LowCutSlope));

    printf("%-25s: %5d (%s)\n", "HighCutSlope",
	   thead.HighCutSlope, printshort(&thead.HighCutSlope));

    printf("%-25s: %5d (%s)\n", "Year",
	   thead.Year, printshort(&thead.Year));

    printf("%-25s: %5d (%s)\n", "DayOfYear",
	   thead.DayOfYear, printshort(&thead.DayOfYear));

    printf("%-25s: %5d (%s)\n", "Hour",
	   thead.Hour, printshort(&thead.Hour));

    printf("%-25s: %5d (%s)\n", "Minute",
	   thead.Minute, printshort(&thead.Minute));

    printf("%-25s: %5d (%s)\n", "Second",
	   thead.Second, printshort(&thead.Second));

    printf("%-25s: %5d (%s)\n", "TimeCode",
	   thead.TimeCode, printshort(&thead.TimeCode));

    printf("%-25s: %5d (%s)\n", "TraceWeightFactor",
	   thead.TraceWeightFactor, printshort(&thead.TraceWeightFactor));

    printf("%-25s: %5d (%s)\n", "GroupRollNumber",
	   thead.GroupRollNumber, printshort(&thead.GroupRollNumber));

    printf("%-25s: %5d (%s)\n", "GroupTraceStart",
	   thead.GroupTraceStart, printshort(&thead.GroupTraceStart));

    printf("%-25s: %5d (%s)\n", "GroupTraceEnd",
	   thead.GroupTraceEnd, printshort(&thead.GroupTraceEnd));

    printf("%-25s: %5d (%s)\n", "GapSize",
	   thead.GapSize, printshort(&thead.GapSize));

    printf("%-25s: %5d (%s)\n", "Overtravel",
	   thead.Overtravel, printshort(&thead.Overtravel));


    printf("%-25s: %5d (%s)\n", "CDPEnsembleX",
	   thead.CDPEnsembleX, printint(&thead.CDPEnsembleX));

    printf("%-25s: %5d (%s)\n", "CDPEnsembleY",
	   thead.CDPEnsembleY, printint(&thead.CDPEnsembleY));

    printf("%-25s: %5d (%s)\n", "InLine",
	   thead.InLine, printint(&thead.InLine));

    printf("%-25s: %5d (%s)\n", "CrossLine",
	   thead.CrossLine, printint(&thead.CrossLine));

    printf("%-25s: %5d (%s)\n", "Shotpoint",
	   thead.Shotpoint, printint(&thead.Shotpoint));

    printf("%-25s: %5d (%s)\n", "ShotpointScalar",
	   thead.ShotpointScalar, printshort(&thead.ShotpointScalar));

    printf("%-25s: %5d (%s)\n", "MeasurementUnit",
	   thead.MeasurementUnit, printshort(&thead.MeasurementUnit));

    printf("%-25s:  ", "TransductionConstant[6]");
    for (i = 0; i < 6; i++) {
      if (i == 0) printf("'");
      printf("%s", printbyte(&thead.TransductionConstant[i]));
    }
    printf("'\n");

    printf("%-25s: %5d (%s)\n", "TransductionUnit",
	   thead.TransductionUnit, printshort(&thead.TransductionUnit));

    printf("%-25s: %5d (%s)\n", "DeviceID",
	   thead.DeviceID, printshort(&thead.DeviceID));

    printf("%-25s: %5d (%s)\n", "TimeScalar",
	   thead.TimeScalar, printshort(&thead.TimeScalar));

    printf("%-25s: %5d (%s)\n", "SourceType",
	   thead.SourceType, printshort(&thead.SourceType));

    printf("%-25s:  ", "SourceDirection[6]");
    for (i = 0; i < 6; i++) {
      if (i == 0) printf("'");
      printf("%hhx", thead.SourceDirection[i]);
      if (i == 5) printf("'\n");
    }

    printf("%-25s:  ", "SourceMeasurement[6]");
    for (i = 0; i < 6; i++) {
      if (i == 0) printf("'");
      printf("%hhx", thead.SourceMeasurement[i]);
      if (i == 5) printf("'\n");
    }

    printf("%-25s: %5d (%s)\n", "SourceMeasurementUnit",
	   thead.SourceMeasurementUnit, printshort(&thead.SourceMeasurementUnit));

    printf("%-25s:  ", "unassigned[8]");
    for (i = 0; i < 8; i++) {
      if (i == 0) printf("'");
      printf("%hhx", thead.unassigned[i]);
      if (i == 7) printf("'\n");
    }

    // print trace data
    printf("\n\nTrace Data %d:\n", trace_num);
    int j;
    for (j = 0; j < 1024; j+=4) {
      printf("%+1.2e %+1.2e %+1.2e %+1.2e ", data[j], data[j+1], 
	     data[j+2], data[j+3]);
      printf("(%s.", printfloat(&data[j]));
      printf("%s.", printfloat(&data[j+1]));
      printf("%s.", printfloat(&data[j+2]));
      printf("%s)\n", printfloat(&data[j+3]));
    }
    printf(" <%9d>\n\n", (int)lseek(fd, 0, SEEK_CUR));    
    if (limit > 0 && trace_num >= limit) break;
  }

  close(fd);
  return 0;
}
